<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Class for URL representation
 *
 * @package ext_bibsonomy_csl
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 * @author Sebastian Böttger
 */
class Tx_ExtBibsonomyCsl_Url {
        
    /**
	 * 
     * 
     * @var array $components 
     */
    protected $components;
    
    /**
     * Flexform settings arrays
	 * 
     * @var array $settings 
     */
    protected $settings;
    
	
	/**
	 * Builds an url from an associatice array like parse_url() returns.
	 * This method is a reproduction from pecl_http method "http_build_url" (http://pecl.php.net/http)
	 * 
	 * The associative components array can have the following keys:
	 * - schmeme (i.e. http)
	 * - user 
	 * - pass
	 * - host (example.com)
	 * - port 
	 * - path (/the/path/)
	 * - query (var1=value1&var2=value2)
	 * - fragment (#chapter1)
	 * 
	 * @param array $components in form of an associative array like parse_url() returns 
	 * @return string URL string
	 */
	public static function http_build_url($components) {
		
		
		$url = $components["scheme"]."://";
		
		if($components["user"] && $components["pass"]) {
			$url .= $components["user"].":".$components["pass"]."@";
		}
		$url .= $components["host"];
		
		if($components["port"]) {
			$url .= ":".$components["port"];
		}
		if($components["path"]) {
			$url .= $components["path"];
		}
		if($components["query"]) {
			$url .= "?".$components["query"];
		}
		if($components["fragment"]) {
			$url .= "#".$components["fragment"];
		}

		return $url;
	}
	
    /**
     * Constructor
	 * 
     * @param mixed array for FlexForm settings or null when using url string
     * @param string $url 
     */
    public function __construct($settings, $url = null) {
        
        
        if($settings != null) {
            $this->settings = $settings;
            $this->initForBibsonomyAPI();
        }
        else if($url != null) {
            $this->components = parse_url($url);
        }
		else {
			throw new Tx_Extbase_Exception("Flexform settings or url needed.");
		}
    }
    
	/**
	 * 
	 */
	public static function initInstanceWithComponents($components) {
		return new self(null, self::http_build_url($components));
	}
	
    /**
	 * Uses the settings array <code>$this->settings</code> with flexform input data sets up an url for
	 * accessing Bibsonomy API. 
	 * 
	 * @throws Tx_ExtBibsonomyCsl_Domain_Exception_AuthenticationException 
	 */
    private function initForBibsonomyAPI() {
        
        //parse baseUrl
        $components = parse_url($this->settings["bib_server"]);
        
        //check authentication
        if( $this->settings["bib_login_name"] == "" || $this->settings["bib_api_key"] == "" ) {
            throw new Tx_ExtBibsonomyCsl_Domain_Exception_AuthenticationException("User name or API key for REST API was not set.");
        }
        /*
        if( $this->settings["bib_api_key"] == "" ) {
            throw new Exception("API key for REST API was not set.");
        }
        */
        //set user/pass for authentication
        $components["user"] = $this->settings["bib_login_name"];
        $components["pass"] = $this->settings["bib_api_key"];
        
        //set path to api
        $components["path"] = "/api/posts";
        
        if(trim($this->settings["bib_custom_api_query"]) != "") {
			$components["query"] = $this->settings["bib_custom_api_query"];
		}
		else {
			//set get query parameters
			$qry = array();

			if($this->settings["bib_content_value"] != "" ) {
				$qry[$this->settings["bib_content_key"]] = $this->settings["bib_content_value"];
			}

			if($this->settings["bib_tags"] != "") {
				$qry["tags"]   = $this->settings["bib_tags"];
			}

			if($this->settings["bib_search"] != "") {
				$qry["search"] = $this->settings["bib_search"];
			}

			$qry["end"] = $this->settings["bib_maxcount"];

			$qry["format"] = "csl";
			$qry["resourcetype"] = "bibtex";
		
			// build query
			$components["query"] = http_build_query($qry);
		}
        
        $this->components = $components;
		
    }
    
    /**
     * returns url string without user and passwort part
     * @return string 
     */
    public function getUrlWithoutAuth() {
        $components = $this->components;
        
        unset($components["user"]);
        unset($components["pass"]);
        
        
        //TODO: replace  http_build_url with own implementation
        return self::http_build_url($components);
    }
    
    
    /**
     * returns user name for authentication
     * @return string 
     */
    public function getAuthUser() {
        return $this->components["user"];
    }
    
    /**
     * returns password for authentication
     * @return string 
     */
    public function getAuthPass() {
        return $this->components["pass"];
    }
    
    /**
	 * returns whole url
     * @return string
     */
    public function getUrl() {
        //TODO: replace  http_build_url with own implementation
        return self::http_build_url($this->components);
    }
    
    /**
     *
     * @return string 
     */
    public function __toString() {
        
        return $this->getUrl();
    }

}

?>
